function [Res1,Res2,Res3] = lbg(action,action2,action3);

% ****************            LBG Algorithmus         ********************
% ************************************************************************
% ***************** Okt./Nov. 2001 von M. Mamsch             *************
% ***************** fr Fehler wird keine Haftung bernommen *************
% ************************************************************************


% ************************ Variablen Definition **************************
% ------------------------------------------------------------------------
% Konstanten:
% -----------
% - AnzahlVektoren			- Anzahl der zu generierenden Merkmalsvektoren (V)
% - KlassenAnzahl					- Anzahl der zu verwendenden Klasssen			  (K)
% - TextAbstand					- Abstand des Textes ber den Vektoren			  
%
% Matrizen:
% ---------
% - Klassenmittelpunkte, 			- Matrix der Gre K x 2 (Fr jede Klasse 2 Koordinaten)
% 	 KlassenmittelpunkteNeu			- Aufbau:
%											- 		Klasse  |  X  |   Y  |
%											- --------------------------
%											-       1     |  2  |   5	|
%											-       2     |  4  |  10  |
%											-      ...	  | ... |  ... |
%											-       K     |  1  |   5  |
% 
% --> Klassenmittelpunkte(K,1) = X-Koordinate der Klasse K
% --> Klassenmittelpunkte(K,2) = Y-Koordinate der Klasse K
%										
% - Testvektoren						- Matrix der Gre V x 2 (Fr jeden Vektor 2 Koordinaten)
%   										- Aufbau, wie Klassenmittelpunkte, jedoch andere Gre
%
% --> Testvektoren (V,1) = X-Koordinate des Vektors V
% --> Testvektoren (V,2) = Y-Koordinate des Vektors V
%
% - Abstaende							- Matrix der Gre V x K  (speichert Abstnde der Vektoren von Klassenmittelpunkten)
%   										- Aufbau:
% 											-    Vektor	|	K1	|	K2	| K3	| ... | Kk	|
%												    1		|	3	|	2	|	5	| ...	| 0.4	|
%												    2		|	1	| 0.3 | 5.2	| ...	| 1	|
%												   ...	| ...	| ...	| ...	| ...	| ... |
%												    V		| 1.2 | 0.1 |  7 	|  2	|  3	|
%
% --> Abstaende(V,K) = Abstands des V-ten Merkmalsvektors von der K-ten Klasse
%
% Vektoren:
% ---------
% - Zaehler(X)				- Zaehlt die Anzahl der in Klasse X klassifizierten Vektoren
%
  
  persistent AnzahlVektoren KlassenAnzahl TextAbstand TestVektoren Klassenmittelpunkte
  persistent Bild DIM Konvergenz GewinnerKlasse KlassenmittelpunkteNeu Button Schritt
  persistent SchrittText Dateiname XMin XMax YMin YMax DimX DimY Check Check2 Anzeige L Zug
  persistent DimXText DimYText OldSelectedVector WaveFile KlassenPlot
  
if nargin < 3, action3 = 10; end;
if nargin < 2, action2 = ''; end;



if nargin < 1
   lbg('Init');
   lbg('Rnd');
   lbg('Initfigure');
   lbg('Paint');
else

	% ------------------------------ Konstanten ------------------------------
	% ------------------------------------------------------------------------
  Buttons = 5;
  ButtonHeight=0.05;
  ButtonWidth=0.7;
  ButtonSpacing=0.01;
  ButtonString={'Step','Run','Reset','Ende','Dim'};
  ButtonStartX=1-(1-ButtonWidth)/2;
  ButtonStartY=1-ButtonHeight;  
  Res1 = []; Res2 = []; Res3 = {};
  
  
  
  if strcmp(action,'Reset')	% Alle Variablen lschen ....   
     try close(Bild); catch end;
     clear;
     lbg('Init');
     
  elseif strcmp(action,'Result')
        Res1 = TestVektoren;
        Res2 = Klassenmittelpunkte;
        Res3 = Zug;
        
  elseif strcmp(action,'Init')
   KlassenAnzahl = action3;     
   if (Bild > 0) close; end;
   Anzeige = 0; 
   Bild = -1;
   
	DimX = 1; DimY = 2;   
   GewinnerKlasse = [];   
   Schritt = 0;
   Konvergenz = 0;
     
  % ************************* Bild erstellen **************************
  elseif strcmp(action,'Initfigure')
   Anzeige = 1;
   Bild = figure('Name','Sprachverarbeitungs Praktikum - LBG Algorithmus', ...
   'NumberTitle','off', ...
   'DoubleBuffer','on', ...
   'Color','white', ...
   'BackingStore','off');
      axes( ...
   'Units','normalized', ...
   'Position',[0.05,0.1+ButtonHeight,0.90-ButtonHeight,0.95-ButtonHeight-0.15]);
   for i = 1 : Buttons,
      Button(i) = uicontrol('Style','push','Units','normalized',...
   'Position',[ButtonStartX-ButtonWidth/Buttons*i-ButtonSpacing*(i-1),ButtonStartY-0.02, ButtonWidth/Buttons, ButtonHeight],...
   'BackgroundColor',[0.70 0.70 0.70],'String',ButtonString{i},...
   'Callback',['lbg(''',ButtonString{i},''')']      );
	end;
	SchrittText = uicontrol('Style','text','Units','normalized',...
   'String','','Position',[0.45,0,0.1,0.1],'BackgroundColor',[1 1 1]);

	DimXText = uicontrol('Style','text','Units','normalized',...
   'String','','Position',[0.9,0,0.1,0.1],'BackgroundColor',[1 1 1]);

	DimYText = uicontrol('Style','text','Units','normalized',...
   'String','','Position',[0.9,0,0.1,0.05],'BackgroundColor',[1 1 1]);




	Check = uicontrol('Style','check','String','Vektornummern anzeigen','Callback','lbg(''Paint'')',...
      					'Position',[20 20 200 20]);
                                    
   set(Check,'Value',0);      
   set([Button(1) Button(2)],'Enable','on');   
   
   set(gcf,'WindowButtonDownFcn','lbg(''ButtonDown'');');
   set(gcf,'KeyPressFcn',...
      'lbg(''KeyPress'');');
      % ************************************************************************************
   
   
   
   % ******************************** Programm durchlaufen  **************************
elseif strcmp(action,'Run')
   Help = Anzeige;
   Anzeige = 0;
   if isempty(Konvergenz), Konvergenz = 0;end;
   
  while Konvergenz == 0,
     lbg('Step');
  end;
  Anzeige = Help;
  lbg('Paint');
   
   
   % ************************* Initiere Zufllige Vektoren erstellen **************************
elseif strcmp(action,'Rnd')
   DIM = 2; 
   AnzahlVektoren = 100;
	% Statusmeldung !
	sprintf('Fhre LBG - Algorithmus mit zuflligen %d Testvektoren und %d Klassen aus !',AnzahlVektoren,KlassenAnzahl)
		
	% -----------------  zufllige Vektoren und Klassen generieren -----------
	TestVektoren = rand(AnzahlVektoren,DIM) * 6;
	Klassenmittelpunkte = rand(KlassenAnzahl,DIM) * 6;
	AnzahlVektoren = 100;   
   XMin = -1; XMax = 7; YMin = -1; YMax = 7;
% ******************************************************************************************

   % ************************* Iterationsschritt berechnen **************************   
elseif strcmp(action,'Step')
   
   [G A Zug] = Klassifizieren(TestVektoren,Klassenmittelpunkte);
	   
	% ---- Nun die neuen Klassenmittelpunkte als Summe der klassifizierten ---
	% ---- Vektoren / Anzahl der Klassifizierten Vektoren berechnen !	     ---
   KlassenmittelpunkteNeu = zeros(KlassenAnzahl,DIM);
   for k = 1:KlassenAnzahl	   
		   if length(Zug{k}) > 0		% Ist ein Vektor in Klasse klassifiziert ???
		      KlassenmittelpunkteNeu(k,:) = VSum(TestVektoren(Zug{k},:)) / length(Zug{k});
		   else	% Falls nicht, bleibt der Klassenmittelpunkt unverndert !
		      KlassenmittelpunkteNeu(k,:) = Klassenmittelpunkte(k,:);
		   end
	end;
			% Ziel erreicht ??? Unterscheiden sich die neuen Klassenmittelpunkte noch von den alten ?   
      if Klassenmittelpunkte == KlassenmittelpunkteNeu,
          if (Anzeige == 1) set([Button(1) Button(2)],'Enable','off'); end;
          Konvergenz = 1;          
      end	   
      Klassenmittelpunkte = KlassenmittelpunkteNeu;
   	Schritt = Schritt+1;
  		GewinnerKlasse = G;
   lbg('Paint');        
% ******************************************************************************************

% ******************************** Neuzeichnen  **************************
elseif strcmp(action,'Paint')
   
   if isempty(Anzeige), Anzeige = 0; end;
   if Anzeige == 1,
   
   hold off;   
   C = GenStrCell([1:AnzahlVektoren],'%d');
   C2=[0:AnzahlVektoren-1];
   if get(Check,'Value'),
      [VX,VY] = VPaint(TestVektoren,'ro',DimX,DimY,C,7,+1,C2);
   else 
      [VX,VY] = VPaint(TestVektoren,'ro',DimX,DimY,GewinnerKlasse,9,-1,C2);
   end;
   
   % ---------- Klassenmittelpunkte zeichnen und beschriften ----------------
   C = GenStrCell([1:KlassenAnzahl],'K%d');
   
   [VX2,VY2] = VPaint(Klassenmittelpunkte,'bx',DimX,DimY,C,9);
	% ------------------------------------------------------------------------
   if Konvergenz == 0 
      set(SchrittText,'String',sprintf('Schritt %d',Schritt));
   else
      set(SchrittText,'String',sprintf('Konvergenz nach Schritt %d',Schritt));
   end;
   
   set(DimXText,'String',sprintf('Dim X: %d',DimX));
   set(DimYText,'String',sprintf('Dim Y: %d',DimY));
   
   XMin = min([VX(1) VX2(1)]);
   XMax = max([VX(2) VX2(2)]);
   
   YMin = min([VY(1) VY2(1)]);
   YMax = max([VY(2) VY2(2)]);
   
   set (gca,'XLim',[XMin,XMax]);
   set (gca,'YLim',[YMin,YMax]);   
	end;	   
% ******************************************************************************************

elseif strcmp(action,'Ende')
   if (Bild > 0) close; end;
   
elseif strcmp(action,'LoadWave'),
   if exist(action2),
      WaveFile = action2;
   end;


% ******************************** Dimensionen neu whlen **************************
elseif strcmp(action,'Dim');
	DimX = str2double(inputdlg('Dimension whlen','Dimension fr X-Achse angeben:'));
   DimY = str2double(inputdlg('Dimension whlen','Dimension fr Y-Achse angeben:'));
   lbg('Paint');
   
elseif strcmp(action,'ButtonDown')
   if exist('OldSelectedVector'), 
      a = findobj('Userdata',OldSelectedVector);
      set(a,'Color',[1 0 0]);
   end;
   x = [];  	
   x = get(gca,'CurrentPoint');
   if not(isempty(x)),
      
	   x = x(1,1:2);
	   x = ([ones(AnzahlVektoren,1) .* x(1),ones(AnzahlVektoren,1) .* x(2)] - ...
   	   [TestVektoren(:,DimX),TestVektoren(:,DimY)]);
      [x, Nr] = min([x(:,1).*x(:,1)+x(:,2).*x(:,2)]);
            
	   a = findobj('UserData',Nr-1);
	   set(a,'Color',[0 1 0]);
      OldSelectedVector = Nr;

      HelpFig = gcf;
      if isempty(KlassenPlot),
          KlassenPlot = figure('Name','SV-Praktikum Klassenplot');
      end;
      figure(KlassenPlot);
      clf;
      if not(isempty(GewinnerKlasse)),
        plot(OldSelectedVector,GewinnerKlasse(OldSelectedVector),'bx');
      end;
      figure(HelpFig);
      
      if not(isempty((WaveFile))), 
         vecplot(Nr-1,20E-3,10E-3,WaveFile);
      end;
   end;
   
   
elseif strcmp(action,'KeyPress')
   x = get(Bild,'CurrentCharacter');
   if isempty(x), return; end;
   
   if strcmp(x,'w')
      if not(isempty(OldSelectedVector)),
	   a = findobj('UserData',OldSelectedVector-1);
	   set(a,'Color',[1 0 0]);       
      OldSelectedVector = min(OldSelectedVector+1,AnzahlVektoren);
      a = findobj('UserData',OldSelectedVector-1);
	   set(a,'Color',[0 1 0]);
      if not(isempty(WaveFile)), 
         vecplot(OldSelectedVector-1,20E-3,10E-3,WaveFile);
      end;
      if not(isempty(KlassenPlot)),
        HelpFig = gcf;
        figure(KlassenPlot);
        hold on;
        
        if not(isempty(GewinnerKlasse)),
            plot([OldSelectedVector-1,OldSelectedVector],[GewinnerKlasse(OldSelectedVector-1),GewinnerKlasse(OldSelectedVector)],'bx');
            plot([OldSelectedVector-1,OldSelectedVector],[GewinnerKlasse(OldSelectedVector-1),GewinnerKlasse(OldSelectedVector)],'r-');
        end;
        hold off;
        figure(HelpFig);
      end;
   end;end;

elseif strcmp(action,'InitVecs')
    lbg('Init');
    
    if not(isempty(action2)),
        TestVektoren = action2;
        
        DIM = size(TestVektoren,2);
        AnzahlVektoren = size(TestVektoren,1);
        
        if isempty(action3),
            KlassenAnzahl = 10;
        else
            KlassenAnzahl = action3;
        end;

      Klassenmittelpunkte = [];
      for kCounter = 1: KlassenAnzahl 
         Klassenmittelpunkte(kCounter,1:DIM) = TestVektoren(floor(rand*AnzahlVektoren+1),:);
      end
    Klassenmittelpunkte;
    end;
    
      
   
   % ******************************** Datei angegeben ? **************************
else
   if exist(action) > 0,
      lbg('Init','',action3);
      [TestVektoren,L] = readvec(action);
   else
      sprintf('Fehler beim Laden der Datei %s',action)
      return;
   end;
   
   Dateiname = action;
   
   if L>0 
      DIM = L; 
      %sprintf('Lade Datei %s !',Dateiname)
      AnzahlVektoren = size(TestVektoren,1);
      
      Klassenmittelpunkte = [];
      for kCounter = 1: KlassenAnzahl 
         Klassenmittelpunkte(kCounter,1:DIM) = TestVektoren(floor(rand*AnzahlVektoren+1),:);
      end
      
      if and(nargin > 1, strcmp(action2,'Paint')),
         Anzeige = 1;
         lbg('Initfigure');
   	end; 
		
      lbg('Paint');
   end;
   
 end;
 
 
end;